﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Pex.Framework;
using Microsoft.Pex.Framework.Settings;
using Microsoft.Pex.Framework.Goals;
using Microsoft.Pex.Framework.Instrumentation;
using UDITA;

[assembly: PexInstrumentAssembly("UDITA")]
[assembly: PexInstrumentAssembly("System.Core")]

namespace IntRedBlackTree
{

    [PexClass(MaxRunsWithoutNewTests = 10000,
              MaxRuns = 20000,
              Timeout = 3600,
              MaxConditions = 2000,
              MaxConstraintSolverTime = 10,
              MaxStack = 1000000000
                                            )]
    public partial class IntRedBlackTreeB2Test
    {
        DateTime time = DateTime.Now;
        TextWriter file = new StreamWriter("IntRedBlackTreeB2Test_log.txt", true);

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testFiltering([PexAssumeNotNull]IntRedBlackTreeB2 rbt, int key)
        {
            try
            {
                PexAssume.IsTrue(rbt.repOK());

                rbt.remove(key);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testCombined(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 6);
                IntRedBlackTreeB2.treeSize = size;
                IntRedBlackTreeB2.maxNodeValue = IntRedBlackTreeB2.treeSize - 1;
                IntRedBlackTreeB2 rbt = new IntRedBlackTreeB2();
                rbt.generateRedBlackTree();
                int key = Verify.getInt(0, IntRedBlackTreeB2.maxNodeValue + 1);

                rbt.remove(key);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Combined time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testObjectPoolFiltering(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 6);
                IntRedBlackTreeB2.treeSize = size;
                IntRedBlackTreeB2.maxNodeValue = IntRedBlackTreeB2.treeSize - 1;
                IntRedBlackTreeB2 rbt = new IntRedBlackTreeB2();
                rbt.generateGraph();
                PexAssume.IsTrue(rbt.repOK());
                int key = Verify.getInt(0, IntRedBlackTreeB2.maxNodeValue + 1);

                rbt.remove(key);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("ObjectPool Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }
    }

    public class IntRedBlackTreeB2 : IntRedBlackTree
    {
        protected override void rightRotate(Node x)
        {
            Node y = x.left;
            x.left = y.right;
            if (y.right != null)
                y.right.parent = x;
            y.parent = x.parent;
            if (x.parent == null)
                root = y;
            else if (x == x.parent.right)
                x.parent.right = y;
            // else // BUG
            // x.parent.left = y; // BUG
            y.right = x;
            x.parent = y;
        }
    }
}