﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Pex.Framework;
using Microsoft.Pex.Framework.Settings;
using Microsoft.Pex.Framework.Goals;
using Microsoft.Pex.Framework.Instrumentation;
using UDITA;

[assembly: PexInstrumentAssembly("UDITA")]
[assembly: PexInstrumentAssembly("System.Core")]

namespace IntRedBlackTree
{

    [PexClass(MaxRunsWithoutNewTests = 10000,
              MaxRuns = 20000,
              Timeout = 3600,
              MaxConditions = 2000,
              MaxConstraintSolverTime = 10,
              MaxStack = 1000000000
                                            )]
    public partial class IntRedBlackTreeB4Test
    {
        DateTime time = DateTime.Now;
        TextWriter file = new StreamWriter("IntRedBlackTreeB4Test_log.txt", true);

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testFiltering([PexAssumeNotNull]IntRedBlackTreeB4 rbt, int key)
        {
            try
            {
                PexAssume.IsTrue(rbt.repOK());

                rbt.remove(key);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testCombined(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 3);
                IntRedBlackTreeB4.treeSize = size;
                IntRedBlackTreeB4.maxNodeValue = IntRedBlackTreeB4.treeSize - 1;
                IntRedBlackTreeB4 rbt = new IntRedBlackTreeB4();
                rbt.generateRedBlackTree();
                int key = Verify.getInt(0, IntRedBlackTreeB4.maxNodeValue + 1);

                rbt.remove(key);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Combined time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testObjectPoolFiltering(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 3);
                IntRedBlackTreeB4.treeSize = size;
                IntRedBlackTreeB4.maxNodeValue = IntRedBlackTreeB4.treeSize - 1;
                IntRedBlackTreeB4 rbt = new IntRedBlackTreeB4();
                rbt.generateGraph();
                PexAssume.IsTrue(rbt.repOK());
                int key = Verify.getInt(0, IntRedBlackTreeB4.maxNodeValue + 1);

                rbt.remove(key);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("ObjectPool Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }
    }

    public class IntRedBlackTreeB4 : IntRedBlackTree
    {
        protected override Node treeDelete(Node z)
        {
            if (z == null)
                return null;

            Node x, y;
            if (z.left == null || z.right == null)
                y = z;
            else
                y = getIOS(z);

            if (y.left != null)
                x = y.left;
            else
                x = y.right;

            if (x != null)
                x.parent = y.parent;

            if (y.parent == null)
                root = x;
            else if (y == y.parent.left)
                y.parent.left = x;
            else
                y.parent.right = x;

            if (y != z)
            {
                z.key = y.key;
                z.value = y.value;
            }

            if (getColor(y) == BLACK)
                if (x == null)
                    // hack
                    ; // BUG
                // treeDeleteFix(y); // BUG
                else
                    ; // BUG
            // treeDeleteFix(x); // BUG
            return y;
        }
    }
}