﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Pex.Framework;
using Microsoft.Pex.Framework.Settings;
using Microsoft.Pex.Framework.Goals;
using Microsoft.Pex.Framework.Instrumentation;
using UDITA;

[assembly: PexInstrumentAssembly("UDITA")]
[assembly: PexInstrumentAssembly("System.Core")]

namespace IntRedBlackTree
{

    [PexClass(MaxRunsWithoutNewTests = 10000,
              MaxRuns = 20000,
              Timeout = 3600,
              MaxConditions = 2000,
              MaxConstraintSolverTime = 10,
              MaxStack = 1000000000
                                            )]
    public partial class IntRedBlackTreeB5Test
    {
        DateTime time = DateTime.Now;
        TextWriter file = new StreamWriter("IntRedBlackTreeB5Test_log.txt", true);

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testFiltering([PexAssumeNotNull]IntRedBlackTreeB5 rbt, int key)
        {
            try
            {
                PexAssume.IsTrue(rbt.repOK());

                rbt.put(key, null);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testCombined(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 3);
                IntRedBlackTreeB5.treeSize = size;
                IntRedBlackTreeB5.maxNodeValue = IntRedBlackTreeB5.treeSize - 1;
                IntRedBlackTreeB5 rbt = new IntRedBlackTreeB5();
                rbt.generateRedBlackTree();
                int key = Verify.getInt(-1, IntRedBlackTreeB5.maxNodeValue + 2);

                rbt.put(key, null);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Combined time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testObjectPoolFiltering(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 3);
                IntRedBlackTreeB5.treeSize = size;
                IntRedBlackTreeB5.maxNodeValue = IntRedBlackTreeB5.treeSize - 1;
                IntRedBlackTreeB5 rbt = new IntRedBlackTreeB5();
                rbt.generateGraph();
                PexAssume.IsTrue(rbt.repOK());
                int key = Verify.getInt(-1, IntRedBlackTreeB5.maxNodeValue + 2);

                rbt.put(key, null);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("ObjectPool Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }
    }

    public class IntRedBlackTreeB5 : IntRedBlackTree
    {
        protected override void treeInsertFix(Node z)
        {
            while (getColor(z.parent) == RED)
            {
                if (parent(z) == left(parent(parent(z))))
                {
                    Node y = right(parent(parent(z)));
                    if (getColor(y) == RED)
                    {
                        setColor(parent(z), BLACK);
                        setColor(y, BLACK);
                        setColor(parent(parent(z)), RED);
                        z = parent(parent(z));
                    }
                    else
                    {
                        if (z == right(parent(z)))
                        {
                            z = parent(z);
                            leftRotate(z);
                        }
                        setColor(parent(z), BLACK);
                        setColor(parent(parent(z)), RED);
                        if (parent(parent(z)) != null)
                            rightRotate(parent(parent(z)));
                    }
                }
                else
                {
                    // Node y = left(parent(parent(z))); // BUG
                    Node y = left(parent(parent(parent(z)))); // BUG
                    if (getColor(y) == RED)
                    {
                        setColor(parent(z), BLACK);
                        setColor(y, BLACK);
                        setColor(parent(parent(z)), RED);
                        z = parent(parent(z));
                    }
                    else
                    {
                        if (z == left(parent(z)))
                        {
                            z = parent(z);
                            rightRotate(z);
                        }
                        setColor(parent(z), BLACK);
                        setColor(parent(parent(z)), RED);
                        if (parent(parent(z)) != null)
                            leftRotate(parent(parent(z)));
                    }
                }
            }
            root.color = BLACK;
        }
    }
}