﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Pex.Framework;
using Microsoft.Pex.Framework.Settings;
using Microsoft.Pex.Framework.Goals;
using Microsoft.Pex.Framework.Instrumentation;
using UDITA;

[assembly: PexInstrumentAssembly("UDITA")]
[assembly: PexInstrumentAssembly("System.Core")]

namespace IntRedBlackTree
{

    [PexClass(MaxRunsWithoutNewTests = 10000,
              MaxRuns = 20000,
              Timeout = 3600,
              MaxConditions = 2000,
              MaxConstraintSolverTime = 10,
              MaxStack = 1000000000
                                            )]
    public partial class IntRedBlackTreeB6Test
    {
        DateTime time = DateTime.Now;
        TextWriter file = new StreamWriter("IntRedBlackTreeB6Test_log.txt", true);

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testFiltering([PexAssumeNotNull]IntRedBlackTreeB6 rbt, int key)
        {
            try
            {
                PexAssume.IsTrue(rbt.repOK());

                rbt.put(key, null);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testCombined(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 1);
                IntRedBlackTreeB6.treeSize = size;
                IntRedBlackTreeB6.maxNodeValue = IntRedBlackTreeB6.treeSize - 1;
                IntRedBlackTreeB6 rbt = new IntRedBlackTreeB6();
                rbt.generateRedBlackTree();
                int key = Verify.getInt(-1, IntRedBlackTreeB6.maxNodeValue + 2);

                rbt.put(key, null);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("Combined time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }

        [PexMethod(TestEmissionFilter = PexTestEmissionFilter.FailuresAndUniquePaths)]
        [PexExpectedGoals]
        public void testObjectPoolFiltering(int size)
        {
            try
            {
                PexAssume.IsTrue(size >= 1 && size <= 1);
                IntRedBlackTreeB6.treeSize = size;
                IntRedBlackTreeB6.maxNodeValue = IntRedBlackTreeB6.treeSize - 1;
                IntRedBlackTreeB6 rbt = new IntRedBlackTreeB6();
                rbt.generateGraph();
                PexAssume.IsTrue(rbt.repOK());
                int key = Verify.getInt(-1, IntRedBlackTreeB6.maxNodeValue + 2);

                rbt.put(key, null);

                PexAssert.IsTrue(rbt.repOK());
            }
            catch (Exception e)
            {
                try
                {
                    file.WriteLine("ObjectPool Filtering time: " + (DateTime.Now - time).TotalMilliseconds);
                    file.Close();
                    PexGoal.Reached();
                }
                catch (Exception)
                {
                }
                throw e;
            }
        }
    }

    public class IntRedBlackTreeB6 : IntRedBlackTree
    {
        protected override void treeInsert(Node z)
        {
            Node y = null;
            Node x = root;
            while (x != null)
            {
                y = x;
                if (z.key.CompareTo(x.key) < 0)
                {
                    x = x.left;
                    size -= 2; // BUG
                }
                else if (z.key.CompareTo(x.key) == 0)
                {
                    x.value = z.value;
                    size--;
                    return;
                }
                else
                    x = x.right;
            }
            z.parent = y;
            if (y == null)
                root = z;
            else if (z.key.CompareTo(y.key) < 0)
                y.left = z;
            else
                y.right = z;
            z.left = null;
            z.right = null;
            z.color = RED;

            treeInsertFix(z);
        }
    }
}